@@ -67,7 +67,19 @@ DirectoryViewContainer::DirectoryViewContainer(QWidget *parent) : QWidget(parent
 
 //    connect(m_proxy, &DirectoryViewProxyIface::menuRequest,
 //            this, &DirectoryViewContainer::menuRequest);
-    connect(m_model, &FileItemModel::changePathRequest, this, &DirectoryViewContainer::updateWindowLocationRequest);
+    connect(m_model, &FileItemModel::changePathRequest, this, &DirectoryViewContainer::signal_responseUnmounted);
+
+    this->setProperty("statusBarUpdate", false);
+    connect(m_model, &FileItemModel::updated, [=](){
+        if(this->property("statusBarUpdate").isValid() && this->property("statusBarUpdate").toBool() == false){
+            this->setProperty("statusBarUpdate", true);
+            QTimer::singleShot(400, this, [=](){
+                Q_EMIT this->statusBarChanged();
+                this->setProperty("statusBarUpdate", false);
+            });
+        }
+    });
+
     connect(FileLabelModel::getGlobalModel(), &FileLabelModel::dataChanged, this, [=](){
         refresh();
     });
@@ -259,6 +271,7 @@ update:
         m_view->beginLocationChange();
         //m_active_view_prxoy->setDirectoryUri(uri);
     }
+    updatePreviewPageRequest();
 }
 
 void DirectoryViewContainer::switchViewType(const QString &viewId)
@@ -290,8 +303,10 @@ void DirectoryViewContainer::switchViewType(const QString &viewId)
     auto oldView = m_view;
     QStringList selection;
     if (oldView) {
-        sortType = oldView->getSortType();
-        sortOrder = oldView->getSortOrder();
+        //fix switch to computer view and back change to default sort issue, link to bug#92261
+        //修复列表视图下改变排序，切换为计算机视图再切回来，排序设置恢复默认方式问题
+//        sortType = oldView->getSortType();
+//        sortOrder = oldView->getSortOrder();
         selection = oldView->getSelections();
         m_layout->removeWidget(dynamic_cast<QWidget*>(oldView));
         oldView->deleteLater();
@@ -338,6 +353,9 @@ void DirectoryViewContainer::switchViewType(const QString &viewId)
     connect(m_view, &DirectoryViewWidget::updateWindowLocationRequest, this, [=](const QString &uri) {
         Q_EMIT this->updateWindowLocationRequest(uri, true);
     });
+    connect(m_view, &DirectoryViewWidget::signal_itemAdded, this, [=](const QString& uri) {
+        Q_EMIT this->signal_itemAdded(uri);
+    });
 
     //m_proxy->switchView(view);
     m_layout->addWidget(dynamic_cast<QWidget*>(view), Qt::AlignBottom);
