@@ -28,6 +28,7 @@
 
 #include "file-item.h"
 #include "file-utils.h"
+#include "file-info.h"
 #include "list-view-style.h"
 
 #include "global-settings.h"
@@ -44,20 +45,39 @@
 #include <QDragEnterEvent>
 #include <QMimeData>
 #include <QDragMoveEvent>
+#include <QDrag>
+#include <QPainter>
+#include <QWindow>
 
 #include <QApplication>
 #include <QStyleHints>
 
 #include <QDebug>
+#include <QToolTip>
+
+#include <QStyleOptionViewItem>
+
+#include <QStandardPaths>
+#include <QMessageBox>
 
 using namespace Peony;
 using namespace Peony::DirectoryView;
 
 ListView::ListView(QWidget *parent) : QTreeView(parent)
 {
+    // use scroll per pixel mode for calculate vertical scroll bar range.
+    // see reUpdateScrollBar()
+    setVerticalScrollMode(ScrollPerPixel);
     setAttribute(Qt::WA_TranslucentBackground);
     setStyle(Peony::DirectoryView::ListViewStyle::getStyle());
 
+    setAutoScroll(true);
+    setAutoScrollMargin(100);
+
+    setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
+
+    setSelectionBehavior(QTreeView::SelectRows);
+
     setAlternatingRowColors(true);
     setAutoFillBackground(true);
     setBackgroundRole(QPalette::Base);
@@ -66,7 +86,7 @@ ListView::ListView(QWidget *parent) : QTreeView(parent)
 
     header()->setSectionResizeMode(QHeaderView::Interactive);
     header()->setSectionsMovable(true);
-    header()->setStretchLastSection(true);
+    //header()->setStretchLastSection(true);
 
     setExpandsOnDoubleClick(false);
     setSortingEnabled(true);
@@ -82,18 +102,40 @@ ListView::ListView(QWidget *parent) : QTreeView(parent)
     m_renameTimer = new QTimer(this);
     m_renameTimer->setInterval(3000);
     m_editValid = false;
+
+    //use this property to fix bug 44314 and 33558
+    //bug#42244 need to find and fix update fail issue
+    setUniformRowHeights(true);
+    setIconSize(QSize(40, 40));
+    setMouseTracking(true);//追踪鼠标
+
+    m_rubberBand = new QRubberBand(QRubberBand::Shape::Rectangle, this);
+
+    //FIXME: do not create proxy in view itself.
+    ListViewDelegate *delegate = new ListViewDelegate(this);
+    setItemDelegate(delegate);
+    connect(delegate, &ListViewDelegate::isEditing, this, [=](const bool &editing)
+    {
+        m_delegate_editing = editing;
+    });
+
+    //fix head indication sort type and order not change in preference file issue, releated to bug#92525,
+    connect(header(), &QHeaderView::sortIndicatorChanged, this, [=](int logicalIndex, Qt::SortOrder order)
+    {
+        //qDebug() << "sortIndicatorChanged:" <<logicalIndex<<order;
+        Peony::GlobalSettings::getInstance()->setValue(SORT_COLUMN, logicalIndex);
+        Peony::GlobalSettings::getInstance()->setValue(SORT_ORDER, order);
+    });
 }
 
 void ListView::scrollTo(const QModelIndex &index, QAbstractItemView::ScrollHint hint)
 {
-    // NOTE:
-    // scrollTo() is confilcted with updateGeometry(), where it will
-    // leave a space for view. So I override this method. However,
-    // the fast keyboard locating of default tree view will be disabled
-    // due to the function is overrided, too.
-
-    //QTreeView::scrollTo(index, hint);
-    //updateGeometries();
+    // Note: due to we rewrite the calculation of view scroll area based on current process,
+    // we could not use QTreeView::scrollTo in some cases because it still based on old process
+    // and will conflict with new calculation.
+    Q_UNUSED(index)
+    Q_UNUSED(hint)
+    reUpdateScrollBar();
 }
 
 bool ListView::isDragging()
@@ -137,7 +179,10 @@ void ListView::bindModel(FileItemModel *sourceModel, FileItemProxyFilterSortMode
 
     //edit trigger
     connect(this->selectionModel(), &QItemSelectionModel::selectionChanged, [=](const QItemSelection &selection, const QItemSelection &deselection) {
-        qDebug()<<"list view selection changed";
+        qDebug()<<"list view selection changed"<<m_delegate_editing;
+        //continue to fix bug#89540，98951
+        if (m_delegate_editing)
+            return;
         auto currentSelections = selection.indexes();
 
         for (auto index : deselection.indexes()) {
@@ -171,8 +216,30 @@ void ListView::bindModel(FileItemModel *sourceModel, FileItemProxyFilterSortMode
 void ListView::keyPressEvent(QKeyEvent *e)
 {
     QTreeView::keyPressEvent(e);
-    if (e->key() == Qt::Key_Control)
+    switch (e->key()) {
+    case Qt::Key_Control:
         m_ctrl_key_pressed = true;
+        break;
+    case Qt::Key_Up: {
+        if (!selectedIndexes().isEmpty()) {
+            QTreeView::scrollTo(selectedIndexes().first());
+        }
+        break;
+    }
+    case Qt::Key_Down: {
+        if (!selectedIndexes().isEmpty()) {
+            auto index = selectedIndexes().first();
+            if (index.row() + 1 == model()->rowCount()) {
+                verticalScrollBar()->setValue(qMin(verticalScrollBar()->value() + iconSize().height(), verticalScrollBar()->maximum()));
+            } else {
+                QTreeView::scrollTo(selectedIndexes().first());
+            }
+        }
+        break;
+    }
+    default:
+        break;
+    }
 }
 
 void ListView::keyReleaseEvent(QKeyEvent *e)
@@ -190,9 +257,14 @@ void ListView::mousePressEvent(QMouseEvent *e)
                 return;
         }
         Q_EMIT customContextMenuRequested(e->pos());
+        m_rubberBand->hide();
         return;
     }
 
+    m_isLeftButtonPressed = true;
+    m_rubberBand->hide();
+    m_lastPressedLogicPoint = e->pos() + QPoint(horizontalOffset(), verticalOffset());
+
     auto index = indexAt(e->pos());
     bool isIndexSelected = selectedIndexes().contains(index);
 
@@ -258,9 +330,41 @@ void ListView::mousePressEvent(QMouseEvent *e)
 void ListView::mouseReleaseEvent(QMouseEvent *e)
 {
     QTreeView::mouseReleaseEvent(e);
+    m_rubberBand->hide();
+    m_isLeftButtonPressed = false;
+}
 
-    if (e->button() != Qt::LeftButton) {
-        return;
+void ListView::mouseMoveEvent(QMouseEvent *e)
+{
+    QModelIndex itemIndex = indexAt(e->pos());
+    if (!itemIndex.isValid()) {
+        if (QToolTip::isVisible()) {
+             QToolTip::hideText();
+         }
+    } else {
+        if (0 != itemIndex.column() && QToolTip::isVisible()) {
+            QToolTip::hideText();
+        }
+    }
+
+    QTreeView::mouseMoveEvent(e);
+
+    if (e->buttons() & Qt::LeftButton) {
+        auto pos = e->pos();
+        auto offset = QPoint(horizontalOffset(), verticalOffset());
+        auto logicPos = pos + offset;
+        QRect logicRect = QRect(logicPos, m_lastPressedLogicPoint);
+        m_logicRect = logicRect.normalized();
+
+        int dx = -horizontalOffset();
+        int dy = -verticalOffset() + this->header()->height();
+        auto realRect = m_logicRect.adjusted(dx, dy, dx ,dy);
+
+        if (!m_rubberBand->isVisible())
+            m_rubberBand->show();
+        m_rubberBand->setGeometry(realRect);
+    } else {
+        m_rubberBand->hide();
     }
 }
 
@@ -276,7 +380,7 @@ void ListView::dragEnterEvent(QDragEnterEvent *e)
     m_editValid = false;
     qDebug()<<"dragEnterEvent()";
     //QTreeView::dragEnterEvent(e);
-    if (e->keyboardModifiers() && Qt::ControlModifier)
+    if (e->keyboardModifiers() & Qt::ControlModifier)
         m_ctrl_key_pressed = true;
     else
         m_ctrl_key_pressed = false;
@@ -284,6 +388,13 @@ void ListView::dragEnterEvent(QDragEnterEvent *e)
     auto action = m_ctrl_key_pressed ? Qt::CopyAction : Qt::MoveAction;
     qDebug()<<"dragEnterEvent()" <<action <<m_ctrl_key_pressed;
     if (e->mimeData()->hasUrls()) {
+        if (FileUtils::containsStandardPath(e->mimeData()->urls())) {
+            e->ignore();
+            if (this == e->source()) {
+                clearSelection();
+            }
+            return;
+        }
         e->setDropAction(action);
         e->accept();
     }
@@ -291,13 +402,13 @@ void ListView::dragEnterEvent(QDragEnterEvent *e)
 
 void ListView::dragMoveEvent(QDragMoveEvent *e)
 {
-    if (e->keyboardModifiers() && Qt::ControlModifier)
+    if (e->keyboardModifiers() & Qt::ControlModifier)
         m_ctrl_key_pressed = true;
     else
         m_ctrl_key_pressed = false;
 
     auto action = m_ctrl_key_pressed ? Qt::CopyAction : Qt::MoveAction;
-    qDebug()<<"list view dragMoveEvent()" <<action <<m_ctrl_key_pressed;
+    //qDebug()<<"list view dragMoveEvent()" <<action <<m_ctrl_key_pressed;
     auto index = indexAt(e->pos());
     if (index.isValid() && index != m_last_index) {
         QHoverEvent he(QHoverEvent::HoverMove, e->posF(), e->posF());
@@ -322,6 +433,13 @@ void ListView::dropEvent(QDropEvent *e)
         case QAbstractItemView::DropIndicatorPosition::OnItem: {
             break;
         }
+        case QAbstractItemView::DropIndicatorPosition::OnViewport: {
+            if (e->keyboardModifiers() & Qt::ControlModifier) {
+                break;
+            } else {
+                return;
+            }
+        }
         default:
             return;
         }
@@ -330,21 +448,49 @@ void ListView::dropEvent(QDropEvent *e)
 
     m_last_index = QModelIndex();
     //m_edit_trigger_timer.stop();
-    if (e->keyboardModifiers() && Qt::ControlModifier)
+    if (e->keyboardModifiers() & Qt::ControlModifier)
         m_ctrl_key_pressed = true;
     else
         m_ctrl_key_pressed = false;
 
     auto action = m_ctrl_key_pressed ? Qt::CopyAction : Qt::MoveAction;
     e->setDropAction(action);
+    if (e->keyboardModifiers() & Qt::ShiftModifier) {
+        action = Qt::TargetMoveAction;
+    }
+
+    //Do not allow dragging files to file manager when searching
+    //related to bug#107063,118004
+    if (m_current_uri.startsWith("search://") || m_current_uri.startsWith("favorite://")) {
+        QMessageBox::warning(this, tr("warn"), tr("This operation is not supported."));
+        return;
+    }
+
     auto proxy_index = indexAt(e->pos());
     auto index = m_proxy_model->mapToSource(proxy_index);
     qDebug()<<"dropEvent" <<action <<m_ctrl_key_pressed <<indexAt(e->pos()).isValid();
+
+    QString username = QStandardPaths::writableLocation(QStandardPaths::HomeLocation).split("/").last();
+    QString boxpath = "file://"+QStandardPaths::writableLocation(QStandardPaths::HomeLocation)+"/.box";
+    QString oldboxpath = "file://box/"+username;
+
+    if(m_current_uri == boxpath || m_current_uri == oldboxpath || m_current_uri == "filesafe:///"){
+        return;
+    }
+
     //move in current path, do nothing
     if (e->source() == this)
     {
         if (indexAt(e->pos()).isValid())
-            m_model->dropMimeData(e->mimeData(), action, 0, 0, index);
+        {
+            auto uri = m_proxy_model->itemFromIndex(proxy_index)->uri();
+            if(!e->mimeData()->urls().contains(uri))
+                m_model->dropMimeData(e->mimeData(), action, 0, 0, index);
+        } else {
+            if (m_ctrl_key_pressed) {
+                m_model->dropMimeData(e->mimeData(), Qt::CopyAction, 0, 0, QModelIndex());
+            }
+        }
         return;
     }
 
@@ -360,19 +506,44 @@ void ListView::resizeEvent(QResizeEvent *e)
     }
 }
 
-void ListView::updateGeometries()
+/*!
+ * \brief ListView::reUpdateScrollBar
+ * \details
+ * tree view use QTreeViewPrivate::updateScrollBars() for reset scrollbar range.
+ * there are 3 parts for that method called.
+ *
+ * 1. QTreeView::scrollTo()
+ * 2. QTreeView::dataChanged()
+ * 3. QTreeView::updateGeometries()
+ *
+ * we have to override all of them to make sure that our custom scrollbar range
+ * set correctly.
+ */
+void ListView::reUpdateScrollBar()
 {
-    QTreeView::updateGeometries();
     if (!model())
         return;
 
-    if (model()->columnCount() == 0 || model()->rowCount() == 0)
+    if (model()->rowCount() == 0) {
         return;
+    }
 
-    QStyleOptionViewItem opt = viewOptions();
-    int height = itemDelegate()->sizeHint(opt, QModelIndex()).height();
-    verticalScrollBar()->setMaximum(verticalScrollBar()->maximum() + 2);
-    //setViewportMargins(0, header()->height(), 0, height);
+    int totalHeight = 0;
+    int rowCount = model()->rowCount();
+    for (int row = 0; row < rowCount; row++) {
+        auto index = model()->index(row, 0);
+        totalHeight += sizeHintForIndex(index).height();
+    }
+
+    verticalScrollBar()->setSingleStep(iconSize().height());
+    verticalScrollBar()->setPageStep(viewport()->height() - header()->height());
+    verticalScrollBar()->setRange(0, totalHeight + header()->height() + 100 - viewport()->height());
+}
+
+void ListView::updateGeometries()
+{
+    QTreeView::updateGeometries();
+    reUpdateScrollBar();
 }
 
 void ListView::wheelEvent(QWheelEvent *e)
@@ -384,13 +555,98 @@ void ListView::wheelEvent(QWheelEvent *e)
     QTreeView::wheelEvent(e);
 }
 
+void ListView::focusInEvent(QFocusEvent *e)
+{
+    QTreeView::focusInEvent(e);
+    if (e->reason() == Qt::TabFocus) {
+        if (selectedIndexes().isEmpty()) {
+            selectionModel()->select(model()->index(0, 0), QItemSelectionModel::SelectCurrent|QItemSelectionModel::Rows);
+        } else {
+            QTreeView::scrollTo(selectedIndexes().first(), QTreeView::EnsureVisible);
+            reUpdateScrollBar();
+            auto selections = getSelections();
+            clearSelection();
+            //use uri rather than index, to fix crash bug#68788, 96145
+            QTimer::singleShot(100, this, [=](){
+                setSelections(selections);
+            });
+        }
+    }
+}
+
+void ListView::startDrag(Qt::DropActions flags)
+{
+    auto indexes = selectedIndexes();
+    if (indexes.count() > 0) {
+        auto pos = mapFromGlobal(QCursor::pos());
+        qreal scale = 1.0;
+        QWidget *window = this->window();
+        if (window) {
+            auto windowHandle = window->windowHandle();
+            if (windowHandle) {
+                scale = windowHandle->devicePixelRatio();
+            }
+        }
+
+        auto drag = new QDrag(this);
+        drag->setMimeData(model()->mimeData(indexes));
+
+        QRegion rect;
+        QHash<QModelIndex, QRect> indexRectHash;
+        for (auto index : indexes) {
+            rect += (visualRect(index));
+            indexRectHash.insert(index, visualRect(index));
+        }
+
+        QRect realRect = rect.boundingRect();
+        QPixmap pixmap(realRect.size() * scale);
+        pixmap.fill(Qt::transparent);
+        pixmap.setDevicePixelRatio(scale);
+        QPainter painter(&pixmap);
+        for (auto index : indexes) {
+            painter.save();
+            painter.translate(indexRectHash.value(index).topLeft() - rect.boundingRect().topLeft());
+            //painter.translate(-rect.boundingRect().topLeft());
+            QStyleOptionViewItem opt = viewOptions();
+            auto viewItemDelegate = static_cast<ListViewDelegate *>(itemDelegate());
+            viewItemDelegate->initIndexOption(&opt, index);
+            opt.displayAlignment = Qt::Alignment(Qt::AlignLeft|Qt::AlignVCenter);
+            opt.rect.setSize(indexRectHash.value(index).size());
+            opt.rect.moveTo(0, 0);
+            opt.state |= QStyle::State_Selected;
+            painter.setOpacity(0.8);
+            QApplication::style()->drawControl(QStyle::CE_ItemViewItem, &opt, &painter);
+            painter.restore();
+        }
+
+        drag->setPixmap(pixmap);
+        drag->setHotSpot(pos - rect.boundingRect().topLeft() - QPoint(0, header()->height()));
+        drag->setDragCursor(QPixmap(), m_ctrl_key_pressed? Qt::CopyAction: Qt::MoveAction);
+        drag->exec(m_ctrl_key_pressed? Qt::CopyAction: Qt::MoveAction);
+    }
+}
+
 void ListView::slotRename()
 {
+    //special path like trash path not allow rename
+    if (getDirectoryUri().startsWith("trash://")
+        || getDirectoryUri().startsWith("recent://")
+        || getDirectoryUri().startsWith("favorite://")
+        || getDirectoryUri().startsWith("search://"))
+        return;
+
+    //standardPaths not allow rename
+    auto currentSelections = getSelections();
+    bool hasStandardPath = FileUtils::containsStandardPath(currentSelections);
+    if (hasStandardPath)
+        return;
+
     //delay edit action to avoid doubleClick or dragEvent
     qDebug()<<"slotRename"<<m_editValid;
-    QTimer::singleShot(300, m_renameTimer, [&]() {
+    QTimer::singleShot(300, this, [&]() {
         qDebug()<<"singleshot"<<m_editValid;
-        if(m_editValid) {
+        //fix bug#98951, click edit box boarder will reenter edit issue
+        if(m_editValid &&  ! m_delegate_editing) {
             m_renameTimer->stop();
             setIndexWidget(m_last_index, nullptr);
             edit(m_last_index);
@@ -423,22 +679,37 @@ void ListView::adjustColumnsSize()
     if (model()->columnCount() == 0)
         return;
 
-    resizeColumnToContents(0);
+    header()->resizeSections(QHeaderView::ResizeToContents);
 
     int rightPartsSize = 0;
     for (int column = 1; column < model()->columnCount(); column++) {
-        resizeColumnToContents(column);
         int columnSize = header()->sectionSize(column);
         rightPartsSize += columnSize;
     }
 
-    if (this->width() - rightPartsSize < BOTTOM_STATUS_MARGIN)
+    //set column 0 minimum width, fix header icon overlap with name issue
+    if(columnWidth(0) < columnWidth(1))
+        setColumnWidth(0, columnWidth(1));
+
+    if (this->width() - rightPartsSize < BOTTOM_STATUS_MARGIN) {
+        int size = width() - BOTTOM_STATUS_MARGIN;
+        size /= header()->count() - 1;
+        setColumnWidth(0, BOTTOM_STATUS_MARGIN);
+        for (int column = 1; column < model()->columnCount(); column++) {
+            setColumnWidth(column, size);
+        }
         return;
+    }
 
-    //resizeColumnToContents(0);
     header()->resizeSection(0, this->viewport()->width() - rightPartsSize);
 }
 
+void ListView::dataChanged(const QModelIndex &topLeft, const QModelIndex &bottomRight, const QVector<int> &roles)
+{
+    QTreeView::dataChanged(topLeft, bottomRight, roles);
+    reUpdateScrollBar();
+}
+
 DirectoryViewProxyIface *ListView::getProxy()
 {
     return m_proxy;
@@ -459,24 +730,29 @@ void ListView::setDirectoryUri(const QString &uri)
 const QStringList ListView::getSelections()
 {
     QStringList uris;
+    QString uri;
     QModelIndexList selections = selectedIndexes();
     for (auto index : selections) {
         if (index.column() == 0)
             uris<<index.data(FileItemModel::UriRole).toString();
     }
+    uris.removeDuplicates();
     return uris;
 }
 
 void ListView::setSelections(const QStringList &uris)
 {
     clearSelection();
+    QItemSelection selection;
     for (auto uri: uris) {
         const QModelIndex index = m_proxy_model->indexFromUri(uri);
         if (index.isValid()) {
-            auto flags = QItemSelectionModel::Select|QItemSelectionModel::Rows;
-            selectionModel()->select(index, flags);
+            QItemSelection selectionToBeMerged(index, index);
+            selection.merge(selectionToBeMerged, QItemSelectionModel::Select);
         }
     }
+    auto flags = QItemSelectionModel::Select|QItemSelectionModel::Rows;
+    selectionModel()->select(selection, flags);
 }
 
 const QStringList ListView::getAllFileUris()
@@ -484,6 +760,17 @@ const QStringList ListView::getAllFileUris()
     return m_proxy_model->getAllFileUris();
 }
 
+QRect ListView::visualRect(const QModelIndex &index) const
+{
+    auto rect = QTreeView::visualRect(index);
+    //comment to fix rename state not show icon issue
+//    if (index.column() == 0) {
+//        rect.setX(0);
+//    }
+
+    return rect;
+}
+
 void ListView::open(const QStringList &uris, bool newWindow)
 {
     return;
@@ -492,6 +779,7 @@ void ListView::open(const QStringList &uris, bool newWindow)
 void ListView::beginLocationChange()
 {
     m_editValid = false;
+    m_last_index = QModelIndex();
     //setModel(nullptr);
     m_model->setRootUri(m_current_uri);
 }
@@ -518,6 +806,7 @@ void ListView::scrollToSelection(const QString &uri)
 {
     auto index = m_proxy_model->indexFromUri(uri);
     QTreeView::scrollTo(index);
+    reUpdateScrollBar();
 }
 
 void ListView::setCutFiles(const QStringList &uris)
@@ -525,6 +814,11 @@ void ListView::setCutFiles(const QStringList &uris)
     return;
 }
 
+bool ListView::getDelegateEditFlag()
+{
+    return m_delegate_editing;
+}
+
 int ListView::getSortType()
 {
     int type = m_proxy_model->sortColumn();
@@ -533,6 +827,8 @@ int ListView::getSortType()
 
 void ListView::setSortType(int sortType)
 {
+    //fix indicator not agree with actual sort order issue, link to bug#71475
+    header()->setSortIndicator(sortType, Qt::SortOrder(getSortOrder()));
     m_proxy_model->sort(sortType, Qt::SortOrder(getSortOrder()));
 }
 
@@ -543,15 +839,26 @@ int ListView::getSortOrder()
 
 void ListView::setSortOrder(int sortOrder)
 {
+    //fix indicator not agree with actual sort order issue, link to bug#71475
+    header()->setSortIndicator(getSortType(), Qt::SortOrder(sortOrder));
     m_proxy_model->sort(getSortType(), Qt::SortOrder(sortOrder));
 }
 
 void ListView::editUri(const QString &uri)
 {
+    setState(QTreeView::NoState);
     auto origin = FileUtils::getOriginalUri(uri);
-    setIndexWidget(m_proxy_model->indexFromUri(origin), nullptr);
-    QTreeView::scrollTo(m_proxy_model->indexFromUri(origin));
-    edit(m_proxy_model->indexFromUri(origin));
+    if(uri.startsWith("mtp://"))/* Fixbug#82649:在手机内部存储里新建文件/文件夹时，名称不是可编辑状态,都是默认文件名/文件夹名 */
+        origin = uri;
+    QModelIndex index =m_proxy_model->indexFromUri(origin);
+    setIndexWidget(index, nullptr);
+    //注释该行以修复bug:#60474
+//    QTreeView::scrollTo(m_proxy_model->indexFromUri(origin));
+    edit(index);
+    //fix bug#70769, edit box overlapped with status bar issue
+    //qDebug() <<"editUri row"<<m_proxy_model->rowCount()<<index.row();
+    if(index.row() >= m_proxy_model->rowCount()-1)
+       QTreeView::scrollToBottom();
 }
 
 void ListView::editUris(const QStringList uris)
@@ -560,6 +867,31 @@ void ListView::editUris(const QStringList uris)
     //implement batch rename.
 }
 
+void ListView::keyboardSearch(const QString &key)
+{
+    // ensure current index is index in display name column
+    if (currentIndex().column() != 0) {
+        selectionModel()->setCurrentIndex(m_model->index(currentIndex().row(), 0, currentIndex().parent()), QItemSelectionModel::SelectCurrent);
+    }
+
+    // note: checking qtreeview.cpp we can find that the keyboard search only select rows
+    // while selection mode is single selection. so we have a trick here for trigger that
+    // action.
+    setSelectionMode(QTreeView::SingleSelection);
+    QAbstractItemView::keyboardSearch(key);
+    setSelectionMode(QTreeView::ExtendedSelection);
+
+    auto indexes = selectedIndexes();
+    if (!indexes.isEmpty()) {
+        QTreeView::scrollTo(indexes.first(), QTreeView::PositionAtCenter);
+        reUpdateScrollBar();
+        if (verticalScrollBar()->value() < viewport()->height()) {
+            return;
+        }
+        verticalScrollBar()->setValue(qMin(verticalScrollBar()->value() + iconSize().height(), verticalScrollBar()->maximum()));
+    }
+}
+
 //List View 2
 ListView2::ListView2(QWidget *parent) : DirectoryViewWidget(parent)
 {
@@ -590,18 +922,26 @@ void ListView2::bindModel(FileItemModel *model, FileItemProxyFilterSortModel *pr
     m_model = model;
     m_proxy_model = proxyModel;
 
-    m_model->setPositiveResponse(false);
+    //m_model->setPositiveResponse(false);
 
     m_view->bindModel(model, proxyModel);
+    connect(m_model, &FileItemModel::selectRequest, this, &DirectoryViewWidget::updateWindowSelectionRequest);
+    connect(m_model,&FileItemModel::signal_itemAdded, this, [=](const QString& uri){
+        Q_EMIT this->signal_itemAdded(uri);
+    });
     connect(model, &FileItemModel::findChildrenFinished, this, &DirectoryViewWidget::viewDirectoryChanged);
-    connect(m_model, &FileItemModel::updated, m_view, &ListView::resort);
+    //connect(m_model, &FileItemModel::updated, m_view, &ListView::resort);
+    connect(m_model, &FileItemModel::updated, m_view->viewport(), QOverload<>::of(&QWidget::update));
 
     connect(m_view->selectionModel(), &QItemSelectionModel::selectionChanged,
             this, &DirectoryViewWidget::viewSelectionChanged);
 
-    connect(m_view, &ListView::doubleClicked, this, [=](const QModelIndex &index) {
-        qDebug()<<index.data(Qt::UserRole).toString();
-        Q_EMIT this->viewDoubleClicked(index.data(Qt::UserRole).toString());
+    connect(m_view, &ListView::activated, this, [=](const QModelIndex &index) {
+        //when selections is more than 1, let mainwindow to process
+        if (getSelections().count() != 1)
+            return;
+        auto uri = getSelections().first();
+        Q_EMIT this->viewDoubleClicked(uri);
     });
 
     //FIXME: how about multi-selection?
@@ -647,28 +987,31 @@ void ListView2::bindModel(FileItemModel *model, FileItemProxyFilterSortModel *pr
 
     connect(m_model, &FileItemModel::findChildrenFinished, this, [=]() {
         if (m_need_resize_header) {
-            //delay a while for proxy model sorting.
-            QTimer::singleShot(100, this, [=]() {
-                //m_view->setModel(m_proxy_model);
-                //adjust columns layout.
-                m_view->adjustColumnsSize();
-            });
+            m_view->adjustColumnsSize();
         }
         m_need_resize_header = false;
     });
 }
 
+void ListView2::repaintView()
+{
+    m_view->update();
+    m_view->viewport()->update();
+}
+
 void ListView2::setCurrentZoomLevel(int zoomLevel)
 {
     int base = 16;
     int adjusted = base + zoomLevel;
+
     m_view->setIconSize(QSize(adjusted, adjusted));
     m_zoom_level = zoomLevel;
 }
 
 void ListView2::clearIndexWidget()
 {
-    for (auto index : m_view->selectedIndexes()) {
+    for (auto index : m_proxy_model->getAllFileIndexes()) {
         m_view->setIndexWidget(index, nullptr);
+        m_view->closePersistentEditor(index);
     }
 }
